@@ -1,11 +1,11 @@
 /*
-  $Id: document.cxx 15909 2013-04-03 13:52:24Z sloot $
+  $Id: document.cxx 17869 2014-11-26 15:06:19Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/src/document.cxx $
 
-  Copyright (c) 1998 - 2013
+  Copyright (c) 1998 - 2014
   ILK   - Tilburg University
   CLiPS - University of Antwerp
- 
+
   This file is part of libfolia
 
   libfolia is free software; you can redistribute it and/or modify
@@ -38,6 +38,8 @@
 #include <algorithm>
 #include <stdexcept>
 #include "ticcutils/XMLtools.h"
+#include "ticcutils/StringOps.h"
+#include "ticcutils/zipper.h"
 #include "libfolia/document.h"
 #include "config.h"
 
@@ -46,7 +48,19 @@ using namespace TiCC;
 
 namespace folia {
 
-  const string FOLIAVERSION = "0.8.2"; // the FoLiA version we implement
+  void initMT(){
+    try {
+      Document doc( "string='<?xml version=\"1.0\" encoding=\"UTF-8\"?><FoLiA xmlns=\"http://ilk.uvt.nl/folia\" xml:id=\"ID\"><text xml:id=\"text\"><div xml:id=\"head\"><w xml:id=\"w1\"><t>test</t></w></div></text></FoLiA>'" );
+    }
+    catch( exception& e ){
+      cerr << "serious initialization problem: " << e.what() << endl;
+      exit(EXIT_FAILURE);
+    };
+  }
+
+  const int MAJOR_VERSION = 0;
+  const int MINOR_VERSION = 11;
+  const int SUB_VERSION   = 2;
   const string NSFOLIA = "http://ilk.uvt.nl/folia";
   const string NSDCOI = "http://lands.let.ru.nl/projects/d-coi/ns/1.0";
   const string NSIMDI = "http://www.mpi.nl/IMDI/Schema/IMDI";
@@ -62,6 +76,12 @@ namespace folia {
       foliadoc = new FoLiA( this, args );
   }
 
+  string versionstring(){
+    stringstream ss;
+    ss << MAJOR_VERSION << "." << MINOR_VERSION << "." << SUB_VERSION;
+    return ss.str();
+  }
+
   void Document::init(){
     _metadatatype = NATIVE;
     metadata = 0;
@@ -70,7 +90,7 @@ namespace folia {
     _foliaNsIn = 0;
     _foliaNsOut = 0;
     debug = 0;
-    version = FOLIAVERSION;
+    version = versionstring();
   }
 
   Document::~Document(){
@@ -92,23 +112,23 @@ namespace folia {
     }
     return true;
   }
-  
+
   bool operator==( const FoliaElement& a1, const FoliaElement& a2){
-    if ( a1._element_id != a2._element_id )
+    if ( a1.element_id() != a2.element_id() )
       return false;
-    if ( a1._id != a2._id )
+    if ( a1.id() != a2.id() )
       return false;
-    if ( a1._set != a2._set )
+    if ( a1.sett() != a2.sett() )
       return false;
-    if ( a1._class != a2._class )
+    if ( a1.cls() != a2.cls() )
       return false;
-    if ( a1._annotator != a2._annotator )
+    if ( a1.annotator() != a2.annotator() )
       return false;
-    if ( a1._annotator_type != a2._annotator_type )
+    if ( a1.annotatortype() != a2.annotatortype() )
       return false;
-    if ( a1.data.size() == a2.data.size() ) {
-      for ( size_t i = 0; i < a1.data.size(); ++i ){
-	if ( *a1.data[i] != *a2.data[i] )
+    if ( a1.size() == a2.size() ) {
+      for ( size_t i = 0; i < a1.size(); ++i ){
+	if ( *a1.index(i) != *a2.index(i) )
 	  return false;
       }
     }
@@ -120,21 +140,26 @@ namespace folia {
     KWargs::const_iterator it = kwargs.find( "debug" );
     if ( it != kwargs.end() )
       debug = stringTo<int>( it->second );
-    else
-      debug = 0;
-    it = kwargs.find( "load" );
+    it = kwargs.find( "mode" );
+    if ( it != kwargs.end() ){
+      mode = it->second;
+      if ( mode != "permissive" ){
+	throw runtime_error( "FoLiA::Document: unsupported mode value: "+mode );
+      }
+    }
+    it = kwargs.find( "external" );
     if ( it != kwargs.end() )
-      loadall = stringTo<bool>( it->second );
+      external = stringTo<bool>( it->second );
     else
-      loadall = true;
+      external = false;
     it = kwargs.find( "version" );
     if ( it != kwargs.end() ){
       version = it->second;
     }
-    else
-      version.clear();
-    
-    it = kwargs.find( "id" );
+    it = kwargs.find( "_id" );
+    if ( it == kwargs.end() ){
+      it = kwargs.find( "id" );
+    }
     if ( it != kwargs.end() ){
       if ( isNCName( it->second ) ){
 	_id = it->second;
@@ -163,12 +188,38 @@ namespace folia {
   }
 
   void Document::addDocIndex( FoliaElement* el, const string& s ){
+    if ( s.empty() ) {
+      return;
+    }
+    // cerr << _id << "-add docindex " << el << " (" << s << ")" << endl;
+    // using TiCC::operator <<;
+    // cerr << "VOOR: " << sindex << endl;
     if ( sindex.find( s ) == sindex.end() ){
       sindex[s] = el;
       iindex.push_back( el );
     }
     else
       throw DuplicateIDError( s );
+    //    cerr << "NA  : " << sindex << endl;
+  }
+
+  void Document::delDocIndex( const FoliaElement* el, const string& s ){
+    if ( s.empty() ) {
+      return;
+    }
+    // cerr << _id << "-del docindex " << el << " (" << s << ")" << endl;
+    // using TiCC::operator <<;
+    // cerr << "VOOR: " << sindex << endl;
+    sindex.erase(s);
+    //    cerr << "NA  : " << sindex << endl;
+    vector<FoliaElement*>::iterator pos = iindex.begin();
+    while( pos != iindex.end() ){
+      if ( *pos == el ){
+	iindex.erase( pos );
+	return;
+      }
+      ++pos;
+    }
   }
 
   static int error_sink(void *mydata, xmlError *error ){
@@ -181,10 +232,22 @@ namespace folia {
   }
 
   bool Document::readFromFile( const string& s ){
+    ifstream is( s.c_str() );
+    if ( !is.good() ){
+      throw runtime_error( "file not found: " + s );
+    }
     if ( xmldoc ){
-      throw runtime_error( "Document is aready initialized" );
+      throw runtime_error( "Document is already initialized" );
       return false;
     }
+    if ( match_back( s, ".bz2" ) ){
+      string buffer = bz2ReadFile( s );
+      return readFromString( buffer );
+    }
+    else if ( match_back( s, ".gz" ) ) {
+      string buffer = gzReadFile( s );
+      return readFromString( buffer );
+    }
     int cnt = 0;
     xmlSetStructuredErrorFunc( &cnt, (xmlStructuredErrorFunc)error_sink );
     xmldoc = xmlReadFile( s.c_str(), 0, XML_PARSE_NOBLANKS );
@@ -206,12 +269,12 @@ namespace folia {
     }
     if ( debug )
       cout << "Failed to read a doc from " << s << endl;
-    return false;
+    throw XmlError( "No XML document read" );
   }
 
   bool Document::readFromString( const string& s ){
     if ( xmldoc ){
-      throw runtime_error( "Document is aready initialized" );
+      throw runtime_error( "Document is already initialized" );
       return false;
     }
     int cnt = 0;
@@ -238,9 +301,14 @@ namespace folia {
     return false;
   }
 
-  ostream& operator<<( ostream& os, const Document& d ){
-    string s = d.toXml();
-    os << s << endl;
+  ostream& operator<<( ostream& os, const Document *d ){
+    if ( d ){
+      string s = d->toXml();
+      os << s << endl;
+    }
+    else {
+      os << "MISSING DOCUMENT" << endl;
+    }
     return os;
   }
 
@@ -251,6 +319,14 @@ namespace folia {
   }
 
   bool Document::save( const string& fn, const string& nsLabel, bool kanon ) {
+    if ( match_back( fn, ".bz2" ) ){
+      string s = toXml( nsLabel, kanon );
+      return bz2WriteFile( fn, s );
+    }
+    else  if ( match_back( fn, ".gz" ) ){
+      string s = toXml( nsLabel, kanon );
+      return gzWriteFile( fn, s );
+    }
     ofstream os( fn.c_str() );
     if ( os.good() ) {
       return save( os, nsLabel, kanon );
@@ -261,7 +337,7 @@ namespace folia {
 
   int Document::size() const {
     if ( foliadoc )
-      return foliadoc->size(); 
+      return foliadoc->size();
     else
       return 0;
   }
@@ -296,13 +372,13 @@ namespace folia {
     }
     return foliadoc->select<Sentence>( excludeSet );
   }
-  
+
   vector<Sentence*> Document::sentenceParts() const {
     static set<ElementType> excludeSet;
     vector<Sentence*> sents = foliadoc->select<Sentence>( excludeSet );
     return sents;
   }
-  
+
   Sentence *Document::sentences( size_t index ) const {
     vector<Sentence*> v = sentences();
     if ( index < v.size() ){
@@ -322,24 +398,9 @@ namespace folia {
   }
 
   vector<Word*> Document::words() const {
-    static set<ElementType> excludeSet;
-    if ( excludeSet.empty() ){
-      excludeSet.insert( Original_t );
-      excludeSet.insert( Suggestion_t );
-      excludeSet.insert( Alternative_t );
-      excludeSet.insert( Chunk_t );
-      excludeSet.insert( SyntacticUnit_t );
-      excludeSet.insert( Coreferences_t );
-      excludeSet.insert( Semroles_t );
-      excludeSet.insert( Entity_t );
-      excludeSet.insert( Headwords_t );
-      excludeSet.insert( TimingLayer_t );
-      excludeSet.insert( DependencyDependent_t );
-      excludeSet.insert( TimeSegment_t );
-    }
-    return foliadoc->select<Word>( excludeSet );
+    return foliadoc->select<Word>( default_ignore_structure );
   }
-  
+
   Word *Document::words( size_t index ) const {
     vector<Word*> v = words();
     if ( index < v.size() ){
@@ -376,14 +437,6 @@ namespace folia {
       throw range_error( "rparagraphs() index out of range" );
   }
 
-  FoliaElement *Document::append( FoliaElement *txt ){
-    if ( !isinstance( txt, Text_t ) )
-      throw runtime_error( "can only append() a Text_t element to a Document" );
-    else {
-      return foliadoc->append( txt );
-    }
-  }
-
   void Document::setimdi( xmlNode *node ){
     xmlNode *n = xPath( node, "//imdi:Session/imdi:Title" );
     if ( n ){
@@ -393,12 +446,12 @@ namespace folia {
     if ( n ){
       _date = XmlContent( n);
     }
-  
+
     n = xPath( node, "//imdi:Source/imdi:Access/imdi:Publisher" );
     if ( n ){
       _publisher = XmlContent( n );
     }
-  
+
     n = xPath( node, "//imdi:Source/imdi:Access/imdi:Availability" );
     if ( n ){
       _license = XmlContent( n );
@@ -408,6 +461,73 @@ namespace folia {
       _language = XmlContent( n );
   }
 
+  void Document::parsemeta( xmlNode *node ){
+    if ( node ){
+      KWargs att = getAttributes( node );
+      string type = att["id"];
+      string val = XmlContent( node );
+      if ( type == "title" )
+	_title = val;
+      else if ( type == "date" )
+	_date = val;
+      else if ( type == "language" )
+	_language = val;
+      else if ( type == "publisher" )
+	_publisher = val;
+      else if ( type == "licence" )
+	_license = val;
+      else if ( _metadatatype != NATIVE ){
+	throw runtime_error( "meta tag with id=" + type
+			     + " requires NATIVE metadataype." );
+      }
+      else if ( meta_atts[type] != "" ){
+	throw runtime_error( "meta tag with id=" + type
+			     + " is defined more then once " );
+      }
+      else
+	meta_atts[type] = val;
+    }
+  }
+
+  void Document::set_metadata( const string& type, const string& value ){
+    if ( type == "title" )
+      _title = value;
+    else if ( type == "date" )
+      _date = value;
+    else if ( type == "language" )
+      _language = value;
+    else if ( type == "publisher" )
+      _publisher = value;
+    else if ( type == "licence" )
+      _license = value;
+    else if ( _metadatatype != NATIVE ){
+      throw runtime_error( "meta tag with id=" + type
+			   + " requires NATIVE metadataype." );
+    }
+    else
+      meta_atts[type] = value;
+  }
+
+  const string Document::get_metadata( const string& type ) const {
+    if ( type == "title" )
+      return _title;
+    else if ( type == "date" )
+      return _date;
+    else if ( type == "language" )
+      return _language;
+    else if ( type == "publisher" )
+      return _publisher;
+    else if ( type == "licence" )
+      return _license;
+    else {
+      map<string,string>::const_iterator it = meta_atts.find( type );
+      if ( it != meta_atts.end() )
+	return it->second;
+      else
+	return "";
+    }
+  }
+
   void Document::parseannotations( xmlNode *node ){
     xmlNode *n = node->children;
     while ( n ){
@@ -445,23 +565,59 @@ namespace folia {
   }
 
   bool checkNS( xmlNode *n, const string& ns ){
-    if ( getNS(n) == ns )
+    string tns = getNS(n);
+    if ( tns == ns )
       return true;
     else
       throw runtime_error( "namespace conflict for tag:" + Name(n)
 			   + ", wanted:" + ns
-			   + " got:" + getNS(n) );
+			   + " got:" + tns );
     return false;
   }
 
+  bool check_version( const string& vers ){
+    vector<string> vec;
+    const int myVersion = MAJOR_VERSION * 10000 + MINOR_VERSION * 100 + SUB_VERSION;
+    int readVersion = 0;
+    split_at( vers, vec, "." );
+    for ( size_t i=0; i < vec.size(); ++i ){
+      int val = stringTo<int>( vec[i] );
+      if ( i == 0 )
+	readVersion += 10000 * val;
+      else if ( i == 1 )
+	readVersion += 100 * val;
+      else
+	readVersion += val;
+    }
+    if ( readVersion <= myVersion )
+      return true;
+    else
+      return false;
+  }
+
+  FoliaElement *Document::resolveExternals( FoliaElement* result ){
+    if ( !externals.empty() ){
+      for ( size_t i= 0;i < externals.size(); ++i ){
+	externals[i]->resolve();
+      }
+    }
+    return result;
+  }
+
   FoliaElement* Document::parseFoliaDoc( xmlNode *root ){
     KWargs att = getAttributes( root );
-    if ( att["id"] == "" ){
+    if ( att["_id"] == "" ){
       throw XmlError("FoLiA Document has no ID!");
       return 0;
     }
+    string vers = att["version"];
+    if ( !check_version( vers ) ){
+      throw XmlError( "FoLiA Document has unsupported version: " + vers
+		      + " (" + version + " is supported.)" );
+      return 0;
+    }
     setAttributes( att );
-    FoliaElement *result = FoliaElement::createElement( this, Name(root) );
+    FoliaElement *result = FoliaImpl::createElement( this, Name(root) );
     if ( debug > 2 )
       cerr << "created " << root << endl;
     result->setAttributes( att );
@@ -503,13 +659,19 @@ namespace folia {
 		cerr << "found annotations" << endl;
 	      parseannotations( m );
 	    }
+	    else if ( Name( m ) == "meta" &&
+		      checkNS( m, NSFOLIA ) ){
+	      if ( debug > 1 )
+		cerr << "found meta node" << endl;
+	      parsemeta( m );
+	    }
 	    m = m->next;
 	  }
 	}
 	else {
 	  if ( p && getNS(p) == NSFOLIA ){
 	    string tag = Name( p );
-	    FoliaElement *t = FoliaElement::createElement( this, tag );
+	    FoliaElement *t = FoliaImpl::createElement( this, tag );
 	    if ( t ){
 	      if ( debug > 2 )
 		cerr << "created " << t << endl;
@@ -525,9 +687,10 @@ namespace folia {
       }
       p = p->next;
     }
+    result = resolveExternals( result );
     return result;
   }
-  
+
   void Document::addStyle( const std::string& type, const std::string& href ){
     if ( type == "text/xsl" ){
       multimap<string,string>::iterator it = styles.find( type );
@@ -536,8 +699,8 @@ namespace folia {
     }
     styles.insert( make_pair( type, href ) );
   }
-  
-  void Document::replaceStyle( const std::string& type, 
+
+  void Document::replaceStyle( const std::string& type,
 			       const std::string& href ){
     multimap<string,string>::iterator it = styles.find( type );
     if ( it != styles.end() ){
@@ -580,6 +743,14 @@ namespace folia {
     }
   }
 
+  void fixupNs( xmlNode *p, xmlNs *ns ){
+    while ( p ){
+      xmlSetNs( p, ns );
+      fixupNs( p->children, ns );
+      p = p->next;
+    }
+  }
+
   FoliaElement* Document::parseXml( ){
     getstyles();
     xmlNode *root = xmlDocGetRootElement( xmldoc );
@@ -593,9 +764,25 @@ namespace folia {
     }
     FoliaElement *result = 0;
     if ( root  ){
-      try {
-	if ( Name( root ) == "FoLiA" &&
-	     checkNS( root, NSFOLIA ) ){
+      if ( Name( root ) == "FoLiA" ){
+	string ns = getNS( root );
+	if ( ns.empty() ){
+	  if ( mode == "permissive" ){
+	    xmlNs *defNs = xmlNewNs( root,
+				     (const xmlChar *)NSFOLIA.c_str(), 0 );
+	    _foliaNsIn = defNs;
+	    fixupNs( root, defNs );
+	  }
+	  else {
+	    throw XmlError( "Folia Document should have namespace declaration "
+			    + NSFOLIA + " but none found " );
+	  }
+	}
+	else if ( ns != NSFOLIA ){
+	  throw XmlError( "Folia Document should have namespace declaration "
+			  + NSFOLIA + " but found: " + ns );
+	}
+	try {
 	  result = parseFoliaDoc( root );
 	  if ( result ){
 	    if ( isNCName( result->id() ) )
@@ -605,22 +792,22 @@ namespace folia {
 	    }
 	  }
 	}
-	else if ( Name( root ) == "DCOI" &&
-		  checkNS( root, NSDCOI ) ){
-	  throw runtime_error( "DCOI format not supported" );
-	}
-	else {
-	  throw runtime_error("root node must be FoLiA" );
+	catch ( exception& e ){
+	  throw XmlError( e.what() );
 	}
       }
-      catch ( exception& e ){
-	throw XmlError( e.what() );
+      else if ( Name( root ) == "DCOI" &&
+		checkNS( root, NSDCOI ) ){
+	throw XmlError( "DCOI format not supported" );
+      }
+      else {
+	throw XmlError( "root node must be FoLiA" );
       }
     }
     return result;
   }
 
-  void Document::declare( AnnotationType::AnnotationType type, 
+  void Document::declare( AnnotationType::AnnotationType type,
 			  const string& setname, const string& args ){
     string st = setname;
     if ( st.empty() )
@@ -648,34 +835,40 @@ namespace folia {
     string res = buf;
     return res;
   }
-  
-  void Document::declare( AnnotationType::AnnotationType type, 
-			  const string& s, const string& a, 
+
+  void Document::declare( AnnotationType::AnnotationType type,
+			  const string& s, const string& a,
 			  const string& t, const string& ds ){
-    string d = ds;
-    if ( d == "now()" ){
-      d = getNow();
-    }
-    if ( !isDeclared( type, s, a, t, d ) ){
+    if ( !isDeclared( type, s, a, t ) ){
+      string d = ds;
+      if ( d == "now()" ){
+	d = getNow();
+      }
       annotationdefaults[type].insert( make_pair(s, at_t(a,t,d) ) );
       //    cerr << "inserted [" << type << "][" << st << "](" << a << "," << t << "," << d ")" << endl;
       //    cerr << "annotation defaults now: " <<  annotationdefaults << endl;
-      
+
     }
   }
 
-  FoliaElement* Document::addNode( ElementType et, const KWargs& kwargs ){
-    FoliaElement *res = FoliaElement::createElement( this, et );
-    res->setAttributes( kwargs );
+  Text* Document::addText( const KWargs& kwargs ){
+    Text *res = new Text( this, kwargs );
     foliadoc->append( res );
     return res;
   }
 
+  Text* Document::addText( Text *t ){
+    foliadoc->append( t );
+    return t;
+  }
+
   bool Document::isDeclared( AnnotationType::AnnotationType type,
-			     const string& s, 
+			     const string& s,
 			     const string& a,
-			     const string& t,
-			     const string& d ){
+			     const string& t){
+    //
+    // We DO NOT check the date. if all parameters match, it is OK
+    //
     if ( type == AnnotationType::NO_ANN ){
       return true;
     }
@@ -685,7 +878,7 @@ namespace folia {
 	throw runtime_error("isDeclared with empty set.");
       multimap<string,at_t>::const_iterator mit2 = mit1->second.lower_bound(s);
       while ( mit2 != mit1->second.upper_bound(s) ){
-	if ( mit2->second.a == a && mit2->second.t == t && mit2->second.d == d )
+	if ( mit2->second.a == a && mit2->second.t == t )
 	  return true;
 	++mit2;
       }
@@ -744,7 +937,7 @@ namespace folia {
       }
       else {
 	if ( mit1->second.count( st ) == 1 ){
-	  map<string,at_t>::const_iterator mit2 = mit1->second.find( st );
+	  multimap<string,at_t>::const_iterator mit2 = mit1->second.find( st );
 	  result = mit2->second.a;
 	}
       }
@@ -768,7 +961,7 @@ namespace folia {
       }
       else {
 	if ( mit1->second.count( st ) == 1 ){
-	  map<string,at_t>::const_iterator mit2 = mit1->second.find( st );
+	  multimap<string,at_t>::const_iterator mit2 = mit1->second.find( st );
 	  result = mit2->second.t;
 	}
       }
@@ -789,7 +982,7 @@ namespace folia {
       }
       else {
 	if ( mit1->second.count( st ) == 1 ){
-	  map<string,at_t>::const_iterator mit2 = mit1->second.find( st );
+	  multimap<string,at_t>::const_iterator mit2 = mit1->second.find( st );
 	  result = mit2->second.d;
 	}
       }
@@ -801,9 +994,9 @@ namespace folia {
   void Document::setannotations( xmlNode *node ) const {
     map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit = annotationdefaults.begin();
     while ( mit != annotationdefaults.end() ){
-      map<string,at_t>::const_iterator it = mit->second.begin();
+      multimap<string,at_t>::const_iterator it = mit->second.begin();
       while ( it != mit->second.end() ){
-	// Find the 'label' 
+	// Find the 'label'
 	string label = toString( mit->first );
 	label += "-annotation";
 	xmlNode *n = XmlNewNode( foliaNs(), label );
@@ -829,29 +1022,68 @@ namespace folia {
   }
 
   void Document::setmetadata( xmlNode *node ) const{
-    KWargs atts;
-    if ( _metadatatype == NATIVE )
-      atts["type"] = "native";
-    else if ( _metadatatype == IMDI )
-      atts["type"] = "imdi";
-    else if ( _metadatatype == CMDI )
-      atts["type"] = "cmdi";
-
     if ( _metadatatype == NATIVE ){
-      if ( !_title.empty() )
-	atts["title"] = _title;
-      if ( !_date.empty() )
-	atts["date"] = _date;
-      if ( !_language.empty() )
-	atts["language"] = _language;
-      if ( !_license.empty() )
-	atts["license"] = _license;
-      if ( !_publisher.empty() )
-	atts["publisher"] = _publisher;
+      KWargs atts;
+      atts["type"] = "native";
       addAttributes( node, atts );
+      if ( !_title.empty() ){
+	xmlNode *m = XmlNewNode( foliaNs(), "meta" );
+	xmlAddChild( m, xmlNewText( (const xmlChar*)_title.c_str()) );
+	KWargs atts;
+	atts["id"] = "title";
+	addAttributes( m, atts );
+	xmlAddChild( node, m );
+      }
+      if ( !_date.empty() ){
+	xmlNode *m = XmlNewNode( foliaNs(), "meta" );
+	xmlAddChild( m, xmlNewText( (const xmlChar*)_date.c_str()) );
+	KWargs atts;
+	atts["id"] = "date";
+	addAttributes( m, atts );
+	xmlAddChild( node, m );
+      }
+      if ( !_language.empty() ){
+	xmlNode *m = XmlNewNode( foliaNs(), "meta" );
+	xmlAddChild( m, xmlNewText( (const xmlChar*)_language.c_str()) );
+	KWargs atts;
+	atts["id"] = "language";
+	addAttributes( m, atts );
+	xmlAddChild( node, m );
+      }
+      if ( !_license.empty() ){
+	xmlNode *m = XmlNewNode( foliaNs(), "meta" );
+	xmlAddChild( m, xmlNewText( (const xmlChar*)_license.c_str()) );
+	KWargs atts;
+	atts["id"] = "license";
+	addAttributes( m, atts );
+	xmlAddChild( node, m );
+      }
+      if ( !_publisher.empty() ){
+	xmlNode *m = XmlNewNode( foliaNs(), "meta" );
+	xmlAddChild( m, xmlNewText( (const xmlChar*)_publisher.c_str()) );
+	KWargs atts;
+	atts["id"] = "publisher";
+	addAttributes( m, atts );
+	xmlAddChild( node, m );
+      }
+      map<string,string>::const_iterator it = meta_atts.begin();
+      while ( it != meta_atts.end() ){
+	xmlNode *m = XmlNewNode( foliaNs(), "meta" );
+	xmlAddChild( m, xmlNewText( (const xmlChar*)it->second.c_str()) );
+	KWargs atts;
+	atts["id"] = it->first;
+	addAttributes( m, atts );
+	xmlAddChild( node, m );
+	++it;
+      }
     }
     else if ( _metadatatype == IMDI  ||
 	      _metadatatype == CMDI ){
+      KWargs atts;
+      if ( _metadatatype == IMDI )
+	atts["type"] = "imdi";
+      else if ( _metadatatype == CMDI )
+	atts["type"] = "cmdi";
       if ( !_metadatafile.empty() )
 	atts["src"] = _metadatafile;
       addAttributes( node, atts );
@@ -862,11 +1094,11 @@ namespace folia {
   void Document::setstyles( xmlDoc* doc ) const {
     multimap<string,string>::const_iterator it = styles.begin();
     while ( it != styles.end() ){
-      string content = "type=\"" + it->first 
+      string content = "type=\"" + it->first
 	+ "\" href=\"" + it->second + "\"";
       xmlAddChild( (xmlNode*)doc,
 		   xmlNewDocPI( doc,
-				(const xmlChar*)"xml-stylesheet", 
+				(const xmlChar*)"xml-stylesheet",
 				(const xmlChar*)content.c_str() ) );
       ++it;
     }
@@ -879,7 +1111,7 @@ namespace folia {
       setstyles( outDoc );
       xmlNode *root = xmlNewDocNode( outDoc, 0, (const xmlChar*)"FoLiA", 0 );
       xmlDocSetRootElement( outDoc, root );
-      xmlNs *xl = xmlNewNs( root, (const xmlChar *)"http://www.w3.org/1999/xlink", 
+      xmlNs *xl = xmlNewNs( root, (const xmlChar *)"http://www.w3.org/1999/xlink",
 			    (const xmlChar *)"xlink" );
       xmlSetNs( root, xl );
       if ( !_foliaNsIn ){
@@ -899,16 +1131,17 @@ namespace folia {
       attribs["generator"] = string("libfolia-v") + VERSION;
       if ( !version.empty() )
 	attribs["version"] = version;
+      if ( external )
+	attribs["external"] = "yes";
       addAttributes( root, attribs );
 
-      xmlNode *md = xmlAddChild( root, XmlNewNode( foliaNs(), "metadata" ) );  
+      xmlNode *md = xmlAddChild( root, XmlNewNode( foliaNs(), "metadata" ) );
       xmlNode *an = xmlAddChild( md, XmlNewNode( foliaNs(), "annotations" ) );
       setannotations( an );
       setmetadata( md );
-      vector<FoliaElement*>::const_iterator it= foliadoc->data.begin();
-      while ( it != foliadoc->data.end() ){
-	xmlAddChild( root, (*it)->xml( true, kanon ) );
-	++it;
+      for ( size_t i=0; i < foliadoc->size(); ++i ){
+	FoliaElement* el = foliadoc->index(i);
+	xmlAddChild( root, el->xml( true, kanon ) );
       }
       xmlChar *buf; int size;
       xmlDocDumpFormatMemoryEnc( outDoc, &buf, &size, "UTF-8", 1 );
@@ -1007,7 +1240,7 @@ namespace folia {
   }
 
   vector<vector<Word*> > Document::findwords( list<Pattern>& pats,
-						      const string& args ) const {
+					      const string& args ) const {
     size_t prevsize = 0;
     bool start = true;
     bool unsetwildcards = false;
@@ -1027,7 +1260,7 @@ namespace folia {
 	if ( index > 0 && variablewildcards.empty() )
 	  unsetwildcards = true;
 	else {
-	  if ( !variablewildcards.empty() && 
+	  if ( !variablewildcards.empty() &&
 	       variablewildcards != it->variablewildcards() ){
 	    throw runtime_error("If multiple patterns are provided with variable wildcards, then these wildcards must all be in the same positions!");
 	  }
@@ -1061,7 +1294,7 @@ namespace folia {
     return result;
   }
 
-  Pattern::Pattern( const std::vector<std::string>&v, 
+  Pattern::Pattern( const std::vector<std::string>&v,
 		    const ElementType at,
 		    const string& args ):matchannotation(at) {
     regexp = false;
@@ -1099,10 +1332,10 @@ namespace folia {
     }
   }
 
-  Pattern::Pattern( const std::vector<std::string>&v, 
+  Pattern::Pattern( const std::vector<std::string>&v,
 		    const string& args ):matchannotation(BASE) {
 
-    // code duplication 
+    // code duplication
     // BAD BAD BAD
     //
     regexp = false;
@@ -1141,7 +1374,7 @@ namespace folia {
   }
 
   Pattern::~Pattern(){
-    for ( size_t i=0; i < matchers.size(); ++i ){  
+    for ( size_t i=0; i < matchers.size(); ++i ){
       delete matchers[i];
     }
   }
@@ -1152,7 +1385,7 @@ namespace folia {
     return os;
   }
 
-  bool Pattern::match( const UnicodeString& us, size_t& pos, int& gap, 
+  bool Pattern::match( const UnicodeString& us, size_t& pos, int& gap,
 		       bool& done, bool& flag ) const {
     UnicodeString s = us;
     //  cerr << "gap = " << gap << "cursor=" << pos << " vergelijk '" <<  sequence[pos] << "' met '" << us << "'" << endl;
