@@ -1,11 +1,11 @@
 /*
-  $Id: foliautils.cxx 15909 2013-04-03 13:52:24Z sloot $
+  $Id: foliautils.cxx 17869 2014-11-26 15:06:19Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/src/foliautils.cxx $
 
-  Copyright (c) 1998 - 2013
+  Copyright (c) 1998 - 2014
   ILK   - Tilburg University
   CLiPS - University of Antwerp
- 
+
   This file is part of libfolia
 
   libfolia is free software; you can redistribute it and/or modify
@@ -35,7 +35,7 @@
 #include <set>
 #include <list>
 #include <stdexcept>
-#include <algorithm> 
+#include <algorithm>
 #include "ticcutils/StringOps.h"
 #include "ticcutils/XMLtools.h"
 #include "libfolia/document.h"
@@ -74,70 +74,70 @@ namespace folia {
     switch ( at ) {
     case AnnotationType::NO_ANN:
       result = "NoNe";
-      break; 
+      break;
     case AnnotationType::TEXT:
       result = "text";
-      break; 
+      break;
     case AnnotationType::STRING:
       result = "string";
-      break; 
+      break;
     case AnnotationType::TOKEN:
       result = "token";
-      break; 
+      break;
     case AnnotationType::DIVISION:
-      result = "div";
-      break; 
-    case AnnotationType::PARAGRAPH: 
+      result = "division"; //adapted from erroneous div
+      break;
+    case AnnotationType::PARAGRAPH:
       result = "paragraph";
-      break; 
-    case AnnotationType::LIST: 
+      break;
+    case AnnotationType::LIST:
       result = "list";
-      break; 
-    case AnnotationType::FIGURE: 
+      break;
+    case AnnotationType::FIGURE:
       result = "figure";
-      break; 
-    case AnnotationType::WHITESPACE: 
+      break;
+    case AnnotationType::WHITESPACE:
       result = "whitespace";
-      break; 
-    case AnnotationType::LINEBREAK: 
+      break;
+    case AnnotationType::LINEBREAK:
       result = "linebreak";
-      break; 
-    case AnnotationType::SENTENCE: 
+      break;
+    case AnnotationType::SENTENCE:
       result = "sentence";
-      break; 
-    case AnnotationType::POS: 
+      break;
+    case AnnotationType::POS:
       result = "pos";
-      break; 
+      break;
     case AnnotationType::LEMMA:
       result = "lemma";
-      break; 
+      break;
     case AnnotationType::DOMEIN:
       result = "domain";
-      break; 
-    case AnnotationType::SENSE: 
+      break;
+    case AnnotationType::SENSE:
       result = "sense";
-      break; 
-    case AnnotationType::SYNTAX: 
+      break;
+    case AnnotationType::SYNTAX:
       result = "syntax";
       break;
-    case AnnotationType::CHUNKING: 
+    case AnnotationType::CHUNKING:
       result = "chunking";
-      break; 
-    case AnnotationType::ENTITY: 
+      break;
+    case AnnotationType::ENTITY:
       result = "entity";
       break;
-    case AnnotationType::CORRECTION: 
+    case AnnotationType::CORRECTION:
       result = "correction";
       break;
-    case AnnotationType::SUGGESTION: 
+    case AnnotationType::SUGGESTION:
       result = "suggestion";
       break;
-    case AnnotationType::ERRORDETECTION: 
+    case AnnotationType::ERRORDETECTION:
       result = "errordetection";
       break;
-    case AnnotationType::ALTERNATIVE: 
+    case AnnotationType::ALTERNATIVE:
       result = "alternative";
-      break; 
+      break;
     case AnnotationType::PHON:
       result = "phon";
       break;
@@ -147,12 +147,12 @@ namespace folia {
     case AnnotationType::MORPHOLOGICAL:
       result = "morphological";
       break;
-    case AnnotationType::SUBENTITY: 
+    case AnnotationType::SUBENTITY:
       result = "subentity";
       break;
     case AnnotationType::EVENT:
       result = "event";
-      break; 
+      break;
     case AnnotationType::DEPENDENCY:
       result = "dependency";
       break;
@@ -161,97 +161,122 @@ namespace folia {
       break;
     case AnnotationType::GAP:
       result = "gap";
-      break; 
+      break;
     case AnnotationType::ALIGNMENT:
       result = "alignment";
-      break; 
+      break;
     case AnnotationType::COMPLEXALIGNMENT:
       result = "complexalignment";
-      break; 
+      break;
     case AnnotationType::COREFERENCE:
       result = "coreference";
-      break; 
+      break;
     case AnnotationType::SEMROLE:
       result = "semrole";
-      break; 
+      break;
     case AnnotationType::METRIC:
       result = "metric";
-      break; 
+      break;
+    case AnnotationType::TABLE:
+      result = "table";
+      break;
+    case AnnotationType::LANG:
+      result = "lang";
+      break;
+    case AnnotationType::STYLE:
+      result = "style";
+      break;
+    case AnnotationType::NOTE:
+      result = "note";
+      break;
+    case AnnotationType::PART:
+      result = "part";
+      break;
     case AnnotationType::LAST_ANN:
       throw logic_error("LAST_ANN");
     };
     return result;
   }
- 
+
   AnnotationType::AnnotationType stringToAT( const string& at ){
     if ( at == "text" )
       return AnnotationType::TEXT;
-    if ( at == "string" )
+    else if ( at == "string" )
       return AnnotationType::STRING;
-    if ( at == "token" )
+    else if ( at == "token" )
       return AnnotationType::TOKEN;
-    if ( at == "div" )
+    else if (( at == "div" ) || ( at == "division")) //patch, documentation and libraries used delse ifferent terms
       return AnnotationType::DIVISION;
-    if ( at == "paragraph" )
+    else if ( at == "paragraph" )
       return AnnotationType::PARAGRAPH;
-    if ( at == "list" )
+    else if ( at == "list" )
       return AnnotationType::LIST;
-    if ( at == "figure" )
+    else if ( at == "figure" )
       return AnnotationType::FIGURE;
-    if ( at == "whitespace" )
+    else if ( at == "whitespace" )
       return AnnotationType::WHITESPACE;
-    if ( at == "linebreak" )
+    else if ( at == "linebreak" )
       return AnnotationType::LINEBREAK;
-    if ( at == "sentence" )
+    else if ( at == "sentence" )
       return AnnotationType::SENTENCE;
-    if ( at == "pos" )
+    else if ( at == "pos" )
       return AnnotationType::POS;
-    if ( at == "lemma" )
+    else if ( at == "lemma" )
       return AnnotationType::LEMMA;
-    if ( at == "domain" )
+    else if ( at == "domain" )
       return AnnotationType::DOMEIN;
-    if ( at == "sense" )
+    else if ( at == "sense" )
       return AnnotationType::SENSE;
-    if ( at == "syntax" )
+    else if ( at == "syntax" )
       return AnnotationType::SYNTAX;
-    if ( at == "chunking" )
+    else if ( at == "chunking" )
       return AnnotationType::CHUNKING;
-    if ( at == "entity" )
+    else if ( at == "entity" )
       return AnnotationType::ENTITY;
-    if ( at == "correction" )
-      return AnnotationType::CORRECTION; 
-    if ( at == "suggestion" )
-      return AnnotationType::SUGGESTION; 
-    if ( at == "errordetection" )
-      return AnnotationType::ERRORDETECTION; 
-    if ( at == "alternative" )
+    else if ( at == "correction" )
+      return AnnotationType::CORRECTION;
+    else if ( at == "suggestion" )
+      return AnnotationType::SUGGESTION;
+    else if ( at == "errordetection" )
+      return AnnotationType::ERRORDETECTION;
+    else if ( at == "alternative" )
       return AnnotationType::ALTERNATIVE;
-    if ( at == "phon" )
+    else if ( at == "phon" )
       return AnnotationType::PHON;
-    if ( at == "subjectivity" )
+    else if ( at == "subjectivity" )
       return AnnotationType::SUBJECTIVITY;
-    if ( at == "morphological" )
+    else if ( at == "morphological" )
       return AnnotationType::MORPHOLOGICAL;
-    if ( at == "subentity" )
+    else if ( at == "subentity" )
       return AnnotationType::SUBENTITY;
-    if ( at == "event" )
+    else if ( at == "event" )
       return AnnotationType::EVENT;
-    if ( at == "dependency" )
+    else if ( at == "dependency" )
       return AnnotationType::DEPENDENCY;
-    if ( at == "timesegment" )
+    else if ( at == "timesegment" )
       return AnnotationType::TIMEDEVENT;
-    if ( at == "gap" )
+    else if ( at == "gap" )
       return AnnotationType::GAP;
-    if ( at == "complexalignment" )
+    else if ( at == "complexalignment" )
       return AnnotationType::COMPLEXALIGNMENT;
-    if ( at == "alignment" )
+    else if ( at == "alignment" )
       return AnnotationType::ALIGNMENT;
-    if ( at == "semrole" )
+    else if ( at == "semrole" )
       return AnnotationType::SEMROLE;
-    if ( at == "coreference" )
+    else if ( at == "coreference" )
       return AnnotationType::COREFERENCE;
-    if ( at == "metric" )
+    else if ( at == "metric" )
       return AnnotationType::METRIC;
+    else if ( at == "table" )
+      return AnnotationType::TABLE;
+    else if ( at == "lang" )
+      return AnnotationType::LANG;
+    else if ( at == "style" )
+      return AnnotationType::STYLE;
+    else if ( at == "note" )
+      return AnnotationType::NOTE;
+    else if ( at == "part" )
+      return AnnotationType::PART;
     throw ValueError( " unknown translation for attribute: " + at );
   }
 
@@ -267,30 +292,38 @@ namespace folia {
     case LineBreak_t: result = "br"; break;
     case WhiteSpace_t: result = "whitespace"; break;
     case Word_t: result = "w"; break;
-    case WordReference_t: result = "wref"; break; 
+    case WordReference_t: result = "wref"; break;
     case Str_t: result = "str"; break;
     case Sentence_t: result = "s"; break;
     case Paragraph_t: result = "p"; break;
     case Division_t: result = "div"; break;
     case Head_t: result = "head"; break;
+    case Table_t: result = "table"; break;
+    case TableHead_t: result = "tablehead"; break;
+    case Cell_t: result = "cell"; break;
+    case Lang_t: result = "lang"; break;
+    case Row_t: result = "row"; break;
     case Caption_t: result = "caption"; break;
     case Label_t: result = "label"; break;
     case List_t: result = "list"; break;
-    case ListItem_t: result = "listitem"; break;
+    case Item_t: result = "item"; break;
     case Figure_t: result = "figure"; break;
     case Quote_t: result = "quote"; break;
+    case TokenAnnotation_t: result = "tokenannotation"; break;
+    case Structure_t: result = "structure"; break;
     case Pos_t: result = "pos"; break;
     case Lemma_t: result = "lemma"; break;
-    case Phon_t: result = "phon"; break; 
-    case Domain_t: result = "domain"; break; 
-    case Sense_t: result = "sense"; break; 
-    case Subjectivity_t: result = "subjectivity"; break; 
+    case Phon_t: result = "phon"; break;
+    case Domain_t: result = "domain"; break;
+    case Sense_t: result = "sense"; break;
+    case Subjectivity_t: result = "subjectivity"; break;
     case Correction_t: result = "correction"; break;
-    case AnnotationLayer_t: result = "annotationlayer"; break; 
-    case SyntacticUnit_t: result = "su"; break; 
-    case SyntaxLayer_t: result = "syntax"; break; 
-    case Chunk_t: result = "chunk"; break; 
-    case Chunking_t: result = "chunking"; break; 
+    case AnnotationLayer_t: result = "annotationlayer"; break;
+    case SpanAnnotation_t: result = "spanannotation"; break;
+    case SyntacticUnit_t: result = "su"; break;
+    case SyntaxLayer_t: result = "syntax"; break;
+    case Chunk_t: result = "chunk"; break;
+    case Chunking_t: result = "chunking"; break;
     case Entity_t: result = "entity"; break;
     case Entities_t: result = "entities"; break;
     case Semroles_t: result = "semroles"; break;
@@ -298,8 +331,6 @@ namespace folia {
     case Coreferences_t: result = "coreferences"; break;
     case CoreferenceLink_t: result = "coreferencelink"; break;
     case CoreferenceChain_t: result = "coreferencechain"; break;
-    // case Subentity_t: result = "subentity"; break;
-    // case Subentities_t: result = "subentities"; break;
     case Morphology_t: result = "morphology"; break;
     case Morpheme_t: result = "morpheme"; break;
     case ErrorDetection_t: result = "errordetection"; break;
@@ -307,8 +338,13 @@ namespace folia {
     case Original_t: result = "original"; break;
     case Current_t: result = "current"; break;
     case Suggestion_t: result = "suggestion"; break;
-    case Alternative_t: result = "alt"; break; 
+    case Alternative_t: result = "alt"; break;
     case Alternatives_t: result = "altlayers"; break;
+    case XmlComment_t: result = "xml-comment"; break;
+    case XmlText_t: result = "xml-text"; break;
+    case External_t: result = "external"; break;
+    case Note_t:result = "note"; break;
+    case Reference_t: result = "ref"; break;
     case Description_t: result = "desc"; break;
     case Gap_t: result = "gap"; break;
     case Content_t: result = "content"; break;
@@ -330,7 +366,14 @@ namespace folia {
     case DependencyDependent_t: result = "dep"; break;
     case Headwords_t: result = "hd"; break;
     case Alignment_t: result = "alignment"; break;
-    case AlignReference_t: result = "aref"; break; 
+    case AlignReference_t: result = "aref"; break;
+    case AbstractTextMarkup_t: result = "textmarkup"; break;
+    case TextMarkupString_t: result = "t-str"; break;
+    case TextMarkupGap_t: result = "t-gap"; break;
+    case TextMarkupCorrection_t: result = "t-correction"; break;
+    case TextMarkupError_t: result = "t-error"; break;
+    case TextMarkupStyle_t: result = "t-style"; break;
+    case Part_t: result = "part"; break;
     default:
       result = "Unknown Elementtype " + folia::toString( int(et) );
     }
@@ -341,230 +384,293 @@ namespace folia {
     if ( tag == "FoLiA" ){
       return BASE;
     }
-    if ( tag == "text" ){
+    else if ( tag == "text" ){
       return Text_t;
     }
-    if ( tag == "w" ){
+    else if ( tag == "w" ){
       return Word_t;
     }
-    if ( tag == "str" ){
+    else if ( tag == "str" ){
       return Str_t;
     }
-    if ( tag == "event" ){
+    else if ( tag == "event" ){
       return Event_t;
     }
-    if ( tag == "timesegment" ){
+    else if ( tag == "timesegment" ){
       return TimeSegment_t;
     }
-    if ( tag == "timing" ){
+    else if ( tag == "timing" ){
       return TimingLayer_t;
     }
-    if ( tag == "s" ){
+    else if ( tag == "s" ){
       return Sentence_t;
     }
-    if ( tag == "t" ){
+    else if ( tag == "t" ){
       return TextContent_t;
     }
-    if ( tag == "br" ){
+    else if ( tag == "br" ){
       return LineBreak_t;
     }
-    if ( tag == "whitespace" ){
+    else if ( tag == "whitespace" ){
       return WhiteSpace_t;
     }
-    if ( tag == "figure" ){
+    else if ( tag == "figure" ){
       return Figure_t;
     }
-    if ( tag == "caption" ){
+    else if ( tag == "caption" ){
       return Caption_t;
     }
-    if ( tag == "label" ){
+    else if ( tag == "label" ){
       return Label_t;
     }
-    if ( tag == "list" ){
+    else if ( tag == "list" ){
       return List_t;
     }
-    if ( tag == "listitem" ){
-      return ListItem_t;
+    else if ( tag == "listitem" || tag == "item" ){
+      return Item_t;
     }
-    if ( tag == "p" ){
+    else if ( tag == "p" ){
       return Paragraph_t;
     }
-    if ( tag == "new" ){
+    else if ( tag == "new" ){
       return New_t;
     }
-    if ( tag == "original" ){
+    else if ( tag == "original" ){
       return Original_t;
     }
-    if ( tag == "current" ){
+    else if ( tag == "current" ){
       return Current_t;
     }
-    if ( tag == "suggestion" ){
+    else if ( tag == "suggestion" ){
       return Suggestion_t;
     }
-    if ( tag == "head" ){
+    else if ( tag == "head" ){
       return Head_t;
     }
-    if ( tag == "desc" ){
+    else if ( tag == "table" ){
+      return Table_t;
+    }
+    else if ( tag == "tablehead" ){
+      return TableHead_t;
+    }
+    else if ( tag == "cell" ){
+      return Cell_t;
+    }
+    else if ( tag == "row" ){
+      return Row_t;
+    }
+    else if ( tag == "lang" ){
+      return Lang_t;
+    }
+    else if ( tag == "xml-comment" ){
+      return XmlComment_t;
+    }
+    else if ( tag == "xml-text" ){
+      return XmlText_t;
+    }
+    else if ( tag == "external" ){
+      return External_t;
+    }
+    else if ( tag == "note" ){
+      return Note_t;
+    }
+    else if ( tag == "ref" ){
+      return Reference_t;
+    }
+    else if ( tag == "desc" ){
       return Description_t;
     }
-    if ( tag == "gap" ){
+    else if ( tag == "gap" ){
       return Gap_t;
     }
-    if ( tag == "content" ){
+    else if ( tag == "content" ){
       return Content_t;
     }
-    if ( tag == "metric" ){
+    else if ( tag == "metric" ){
       return Metric_t;
     }
-    if ( tag == "div" ){
+    else if ( tag == "div" ){
       return Division_t;
     }
-    if ( tag == "annotationlayer" ){
+    else if ( tag == "annotationlayer" ){
       return AnnotationLayer_t;
     }
-    if ( tag == "pos" ){
+    else if ( tag == "spanannotation" ){
+      return SpanAnnotation_t;
+    }
+    else if ( tag == "tokenannotation" ){
+      return TokenAnnotation_t;
+    }
+    else if ( tag == "structure" ){
+      return Structure_t;
+    }
+    else if ( tag == "pos" ){
       return Pos_t;
     }
-    if ( tag == "lemma" ){
+    else if ( tag == "lemma" ){
       return Lemma_t;
     }
-    if ( tag == "phon" ){
+    else if ( tag == "phon" ){
       return Phon_t;
     }
-    if ( tag == "domain" ){
+    else if ( tag == "domain" ){
       return Domain_t;
     }
-    if ( tag == "sense" ){
+    else if ( tag == "sense" ){
       return Sense_t;
     }
-    if ( tag == "syntax" ){
+    else if ( tag == "syntax" ){
       return SyntaxLayer_t;
     }
-    if ( tag == "subjectivity" ){
+    else if ( tag == "subjectivity" ){
       return Subjectivity_t;
     }
-    if ( tag == "chunk" ){
+    else if ( tag == "chunk" ){
       return Chunk_t;
     }
-    if ( tag == "chunking" ){
+    else if ( tag == "chunking" ){
       return Chunking_t;
     }
-    if ( tag == "entity" ){
+    else if ( tag == "entity" ){
       return Entity_t;
     }
-    if ( tag == "entities" ){
+    else if ( tag == "entities" ){
       return Entities_t;
     }
-    if ( tag == "semroles" ){
+    else if ( tag == "semroles" ){
       return Semroles_t;
     }
-    if ( tag == "semrole" ){
+    else if ( tag == "semrole" ){
       return Semrole_t;
     }
-    if ( tag == "coreferences" ){
+    else if ( tag == "coreferences" ){
       return Coreferences_t;
     }
-    if ( tag == "coreferencelink" ){
+    else if ( tag == "coreferencelink" ){
       return CoreferenceLink_t;
     }
-    if ( tag == "coreferencechain" ){
+    else if ( tag == "coreferencechain" ){
       return CoreferenceChain_t;
     }
-    // if ( tag == "subentity" ){
-    //   return Subentity_t;
-    // }
-    // if ( tag == "subentities" ){
-    //   return Subentities_t;
-    // }
-    if ( tag == "alt" ){
+    else if ( tag == "alt" ){
       return Alternative_t;
     }
-    if ( tag == "placeholder" ){
+    else if ( tag == "placeholder" ){
       return PlaceHolder_t;
     }
-    if ( tag == "altlayers" ){
+    else if ( tag == "altlayers" ){
       return Alternatives_t;
     }
-    if ( tag == "su" ){
+    else if ( tag == "su" ){
       return SyntacticUnit_t;
     }
-    if ( tag == "wref" ){
+    else if ( tag == "wref" ){
       return WordReference_t;
     }
-    if ( tag == "correction" ){
+    else if ( tag == "correction" ){
       return Correction_t;
     }
-    if ( tag == "errordetection" ){
+    else if ( tag == "errordetection" ){
       return ErrorDetection_t;
     }
-    if ( tag == "morphology" ){
+    else if ( tag == "morphology" ){
       return Morphology_t;
     }
-    if ( tag == "morpheme" ){
+    else if ( tag == "morpheme" ){
       return Morpheme_t;
     }
-    if ( tag == "feat" ){
+    else if ( tag == "feat" ){
       return Feature_t;
     }
-    if ( tag == "begindatetime" ){
+    else if ( tag == "begindatetime" ){
       return BeginDateTimeFeature_t;
     }
-    if ( tag == "enddatetime" ){
+    else if ( tag == "enddatetime" ){
       return EndDateTimeFeature_t;
     }
-    if ( tag == "synset" ){
+    else if ( tag == "synset" ){
       return SynsetFeature_t;
     }
-    if ( tag == "actor" ){
+    else if ( tag == "actor" ){
       return ActorFeature_t;
     }
-    if ( tag == "headfeature" ){
+    else if ( tag == "headfeature" ){
       return HeadFeature_t;
     }
-    if ( tag == "value" ){
+    else if ( tag == "value" ){
       return ValueFeature_t;
     }
-    if ( tag == "time" ){
+    else if ( tag == "time" ){
       return TimeFeature_t;
     }
-    if ( tag == "level" ){
+    else if ( tag == "level" ){
       return LevelFeature_t;
     }
-    if ( tag == "function" ){
+    else if ( tag == "function" ){
       return FunctionFeature_t;
     }
-    if ( tag == "modality" ){
+    else if ( tag == "modality" ){
       return ModalityFeature_t;
     }
-    if ( tag == "quote" ){
+    else if ( tag == "quote" ){
       return Quote_t;
     }
-    if ( tag == "dependencies" ){
+    else if ( tag == "dependencies" ){
       return Dependencies_t;
     }
-    if ( tag == "dependency" ){
+    else if ( tag == "dependency" ){
       return Dependency_t;
     }
-    if ( tag == "dep" ){
+    else if ( tag == "dep" ){
       return DependencyDependent_t;
     }
-    if ( tag == "hd" ){
+    else if ( tag == "hd" ){
       return Headwords_t;
     }
-    if ( tag == "alignment" ){
+    else if ( tag == "alignment" ){
       return Alignment_t;
     }
-    if ( tag == "aref" ){
+    else if ( tag == "aref" ){
       return AlignReference_t;
     }
+    else if ( tag == "textmarkup" ){
+      return AbstractTextMarkup_t;
+    }
+    else if ( tag == "t-str" ){
+      return TextMarkupString_t;
+    }
+    else if ( tag == "t-gap" ){
+      return TextMarkupGap_t;
+    }
+    else if ( tag == "t-correction" ){
+      return TextMarkupCorrection_t;
+    }
+    else if ( tag == "t-error" ){
+      return TextMarkupError_t;
+    }
+    else if ( tag == "t-style" ){
+      return TextMarkupStyle_t;
+    }
+    else if ( tag == "part" ){
+      return Part_t;
+    }
     else {
       throw ValueError( "unknown tag <" + tag + ">" );
     }
     return BASE;
   }
 
-  FoliaElement *FoliaElement::createElement( Document *doc, 
-					     ElementType et ){
+  FoliaElement *FoliaImpl::createElement( Document *doc,
+					  const string& tag ){
+
+    ElementType et = BASE;
+    try {
+      et = stringToET( tag );
+    }
+    catch ( ValueError& e ){
+      cerr << e.what() << endl;
+      return 0;
+    }
     switch ( et ){
     case BASE:
       return new FoLiA( doc );
@@ -596,8 +702,8 @@ namespace folia {
       return new Label( doc );
     case List_t:
       return new List( doc );
-    case ListItem_t:
-      return new ListItem( doc );
+    case Item_t:
+      return new Item( doc );
     case Paragraph_t:
       return new Paragraph( doc );
     case New_t:
@@ -610,6 +716,32 @@ namespace folia {
       return new Suggestion( doc );
     case Head_t:
       return new Head( doc );
+    case Table_t:
+      return new Table( doc );
+    case TableHead_t:
+      return new TableHead( doc );
+    case Cell_t:
+      return new Cell( doc );
+    case Row_t:
+      return new Row( doc );
+    case Lang_t:
+      return new LangAnnotation( doc );
+    case SpanAnnotation_t:
+      return new AbstractSpanAnnotation( doc );
+    case TokenAnnotation_t:
+      return new AbstractTokenAnnotation( doc );
+    case Structure_t:
+      return new AbstractStructureElement( doc );
+    case XmlComment_t:
+      return new XmlComment( doc );
+    case XmlText_t:
+      return new XmlText( doc );
+    case External_t:
+      return new External( doc );
+    case Note_t:
+      return new Note( doc );
+    case Reference_t:
+      return new Reference( doc );
     case Description_t:
       return new Description( doc );
     case Gap_t:
@@ -654,10 +786,6 @@ namespace folia {
       return new CoreferenceLink( doc );
     case CoreferenceChain_t:
       return new CoreferenceChain( doc );
-    // case Subentity_t:
-    //   return new Subentity( doc );
-    // case Subentities_t:
-    //   return new SubentitiesLayer( doc );
     case Alternative_t:
       return new Alternative( doc );
     case PlaceHolder_t:
@@ -712,25 +840,26 @@ namespace folia {
       return new Alignment( doc );
     case AlignReference_t:
       return new AlignReference( doc );
+    case AbstractTextMarkup_t:
+      return new AbstractTextMarkup( doc );
+    case TextMarkupString_t:
+      return new TextMarkupString( doc );
+    case TextMarkupGap_t:
+      return new TextMarkupGap( doc );
+    case TextMarkupCorrection_t:
+      return new TextMarkupCorrection( doc );
+    case TextMarkupError_t:
+      return new TextMarkupError( doc );
+    case TextMarkupStyle_t:
+      return new TextMarkupStyle( doc );
+    case Part_t:
+      return new Part( doc );
     default:
       throw ValueError( "unknown elementtype(" + toString(int(et)) + ")" );
     }
     return 0;
   }
 
-  FoliaElement *FoliaElement::createElement( Document *doc, 
-					     const string& tag ){
-    
-    ElementType et = BASE;
-    try {
-      et = stringToET( tag );
-    }
-    catch ( ValueError& e ){
-      cerr << e.what() << endl;
-      return 0;
-    }
-    return createElement( doc, et );
-  }
 
   KWargs getArgs( const std::string& s ){
     KWargs result;
@@ -744,7 +873,7 @@ namespace folia {
 
     for ( size_t i=0; i < s.size(); ++i ){
       //      cerr << "bekijk " << s[i] << endl;
-      //      cerr << "quoted = " << (quoted?"YES":"NO") 
+      //      cerr << "quoted = " << (quoted?"YES":"NO")
       // 	   << " parseatt = " << (parseatt?"YES":"NO")
       // 	   << " escaped = " << (escaped?"YES":"NO") << endl;
       if ( s[i] == '\\' ){
@@ -760,7 +889,7 @@ namespace folia {
 	  }
 	}
 	else
-	  throw ArgsError( s + ", stray \\" );	  
+	  throw ArgsError( s + ", stray \\" );
       }
       else if ( s[i] == '\'' ){
 	//	cerr << "handle single quote " << endl;
@@ -841,28 +970,40 @@ namespace folia {
     if ( node ){
       xmlAttr *a = node->properties;
       while ( a ){
-	atts[(char*)a->name] = (char *)a->children->content;
+	if ( a->atype == XML_ATTRIBUTE_ID && string((char*)a->name) == "id" ){
+	  atts["_id"] = (char *)a->children->content;
+	}
+	else
+	  atts[(char*)a->name] = (char *)a->children->content;
 	a = a->next;
       }
     }
     return atts;
   }
 
-  void addAttributes( xmlNode *node, const KWargs& attribs ){
-    KWargs::const_iterator it = attribs.begin();
+  void addAttributes( xmlNode *node, const KWargs& atts ){
+    KWargs attribs = atts;
+    KWargs::iterator it = attribs.find("_id");
+    if ( it != attribs.end() ){ // _id is special for xml:id
+      xmlSetProp( node, XML_XML_ID, (const xmlChar *)it->second.c_str() );
+      attribs.erase(it);
+    }
+    it = attribs.find("lang");
+    if ( it != attribs.end() ){ // lang is special too
+      xmlNodeSetLang( node, (const xmlChar*)it->second.c_str() );
+      attribs.erase(it);
+    }
+    it = attribs.find("id");
+    if ( it != attribs.end() ){ // id is te be sorted before the rest
+      xmlSetProp( node, (const xmlChar*)("id"), (const xmlChar *)it->second.c_str() );
+      attribs.erase(it);
+    }
+    // and now the rest
+    it = attribs.begin();
     while ( it != attribs.end() ){
-      //    cerr << "addAttributes(" << it->first << ", " << it->second << ")" << endl;
-      if ( it->first == "_id" ){ // id is special
-	xmlSetProp( node, XML_XML_ID, (const xmlChar *)it->second.c_str() );
-      }
-      else if ( it->first == "lang" ){ // lang is special
-	xmlNodeSetLang( node, (const xmlChar*)it->second.c_str() );
-      }
-      else {
-	xmlSetProp( node,
-		    (const xmlChar*)it->first.c_str(), 
-		    (const xmlChar*)it->second.c_str() );
-      }
+      xmlSetProp( node,
+		  (const xmlChar*)it->first.c_str(),
+		  (const xmlChar*)it->second.c_str() );
       ++it;
     }
   }
@@ -989,13 +1130,13 @@ namespace folia {
 	  stringTo<AnnotationType::AnnotationType>( s );
 	}
 	catch ( ValueError& e ){
-	  cerr << "no AnnotationType found for string '" << s << "'" << endl;	
+	  cerr << "no AnnotationType found for string '" << s << "'" << endl;
 	  sane = false;
 	}
       }
     }
     return sane;
-  };
+  }
 
   bool ET_sanity_check(){
     bool sane = true;
@@ -1017,14 +1158,14 @@ namespace folia {
 	  continue;
 	}
 	if ( et != et2 ){
-	  cerr << "Argl: toString(ET) doesn't match original:" << s 
+	  cerr << "Argl: toString(ET) doesn't match original:" << s
 	       << " vs " << toString(et2) << endl;
 	  sane = false;
 	  continue;
 	}
-	FoliaElement *tmp = FoliaElement::createElement( 0, s );
+	FoliaElement *tmp = FoliaImpl::createElement( 0, s );
 	if ( tmp == 0 ) {
-	  cerr << "no Element created found for string '" << s << "'" << endl;	
+	  cerr << "no Element created found for string '" << s << "'" << endl;
 	  sane = false;
 	}
 	else {
@@ -1040,26 +1181,26 @@ namespace folia {
       }
     }
     return sane;
-  };
+  }
 
   bool isNCName( const std::string& s ){
-    static string extra=".-_";
+    const string extra=".-_";
     if ( s.empty() )
       throw XmlError( "an empty string is not a valid NCName." );
     else if ( !isalpha(s[0]) )
-      throw XmlError( "'" 
-		      + s 
+      throw XmlError( "'"
+		      + s
 		      + "' is not a valid NCName. (must start with character)." );
     else {
       for ( size_t i=1; i < s.length(); ++i ){
 	if ( !isalnum(s[i]) &&
 	     extra.find(s[i]) == string::npos )
-	  throw XmlError( "'" + s 
-			  + "' is not a valid NCName.(invalid '" 
+	  throw XmlError( "'" + s
+			  + "' is not a valid NCName.(invalid '"
 			  + char(s[i]) + "' found" );
       }
     }
     return true;
   }
-  
+
 } //namespace folia
